package com.simuwang.daq.components.report.parser.pdf;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.exception.APIException;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.*;
import com.simuwang.daq.components.CustomPDFTextStripper;
import com.simuwang.daq.components.report.parser.ReportParserConstant;
import com.smppw.common.pojo.ValueLabelVO;
import org.apache.pdfbox.Loader;
import org.apache.pdfbox.io.RandomAccessReadBufferedFile;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.springframework.stereotype.Component;
import technology.tabula.*;
import technology.tabula.extractors.SpreadsheetExtractionAlgorithm;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2024/9/11 16:19
 * @description pdf格式的月报解析
 */
@Component(ReportParserConstant.PARSER_PDF_MONTHLY)
public class PDMonthlyReportParser extends AbstractReportParser<MonthlyReportData> {
    private final List<Table> extNavTables = ListUtil.list(true);
    private Table baseInfoTable = null;

    public PDMonthlyReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public String getParser() {
        return ReportParserConstant.PARSER_PDF_MONTHLY;
    }

    @Override
    protected String initAndGetReportName(ReportParserParams params) throws IOException {
        String reportName = null;
        try (PDDocument document = Loader.loadPDF(new RandomAccessReadBufferedFile(params.getFilepath()))) {
            CustomPDFTextStripper stripper = new CustomPDFTextStripper();
            stripper.setSortByPosition(true);
            String text = stripper.getText(document);
            text = text.replace("+_+\r\n", "").replace("+_+", "");
            List<String> textList = StrUtil.split(text, "\r\n");
            if (CollUtil.isNotEmpty(textList)) {
                String name = textList.get(0);
                reportName = this.matchReportName(name);
                if (StrUtil.isBlank(reportName)) {
                    throw new APIException("未匹配到报告名称");
                }
            }

            SpreadsheetExtractionAlgorithm extractionAlgorithm = new SpreadsheetExtractionAlgorithm();
            PageIterator pageIterator = new ObjectExtractor(document).extract();
            while (pageIterator.hasNext()) {
                Page page = pageIterator.next();
                List<Table> tables = extractionAlgorithm.extract(page);
                tables = tables.stream().distinct().collect(Collectors.toList());
                for (Table table : tables) {
                    int colCount = table.getColCount();
                    if (colCount == 4) {
                        this.baseInfoTable = table;
                    } else if (colCount >= 5) {
                        this.extNavTables.add(table);
                    }
                }
            }
        }
        return reportName;
    }

    @Override
    protected ReportFundInfoDTO parseBaseInfo(ReportParserParams params) {
        Table baseInfoTable = this.baseInfoTable;
        if (baseInfoTable == null) {
            throw new APIException("未解析到基本信息表格");
        }
        Map<String, Object> baseInfoMap = MapUtil.newHashMap(32);
        for (int i = 0; i < baseInfoTable.getRows().size(); i++) {
            List<RectangularTextContainer> cols = baseInfoTable.getRows().get(i);
            for (int j = 0; j < 2; j++) {
                baseInfoMap.put(cols.get(j * 2).getText(), cols.get(j * 2 + 1).getText());
            }
        }
        // 匹配字段清洗字段
        ReportFundInfoDTO reportFundInfo = new ReportFundInfoDTO();
        this.buildInfo(baseInfoMap, reportFundInfo);
        return reportFundInfo;
    }

    @Override
    protected MonthlyReportData parseExtInfoAndSetData(ReportBaseInfoDTO baseInfo, ReportFundInfoDTO fundInfo, ReportParserParams params) {
        MonthlyReportData reportData = new MonthlyReportData();
        reportData.setBaseInfo(baseInfo);
        reportData.setFundInfo(fundInfo);

        List<ReportNetReportDTO> exts = ListUtil.list(false);
        List<Table> extNavTables = this.extNavTables;
        for (Table extNavTable : extNavTables) {
            Map<String, Object> extInfoMap = MapUtil.newHashMap(16);
            for (int i = 0; i < extNavTable.getColCount(); i++) {
                String key = extNavTable.getCell(0, i).getText();
                String value = extNavTable.getCell(1, i).getText();
                extInfoMap.put(key, value);
            }
            ReportNetReportDTO navInfo = new ReportNetReportDTO();
            buildInfo(extInfoMap, navInfo);
            exts.add(navInfo);
        }
        reportData.setNetReport(exts);
        return reportData;
    }

    private void buildInfo(Map<String, Object> extInfoMap, Object info) {
        for (Map.Entry<String, Object> entry : extInfoMap.entrySet()) {
            String k = entry.getKey();
            Object v = entry.getValue();
            String fieldValue = StrUtil.toStringOrNull(v);
            if (fieldValue.startsWith("-") || fieldValue.endsWith("-")) {
                fieldValue = null;
            }
            if (fieldValue != null) {
                fieldValue = fieldValue.replace("\r", "");
            }
            for (ValueLabelVO vo : this.fieldMapper) {
                String fieldName = vo.getValue();
                List<String> labels = StrUtil.split(vo.getLabel(), ",");
                if (labels.contains(k)) {
                    try {
                        ReflectUtil.setFieldValue(info, fieldName, fieldValue);
                    } catch (Exception e) {
                        this.logger.warn("{} 字段值设置错误：{}", fieldName, e.getMessage());
                    }
                    break;
                }
                for (String label : labels) {
                    if (k.contains(label)) {
                        try {
                            ReflectUtil.setFieldValue(info, fieldName, fieldValue);
                        } catch (Exception e) {
                            this.logger.warn("{} 字段值设置错误：{}", fieldName, e.getMessage());
                        }
                        break;
                    }
                }
            }
        }
    }

    private String matchReportName(String text) {
        if (StrUtil.isBlank(text)) {
            return null;
        }
        // 编译正则表达式模式
        Pattern pat1 = Pattern.compile(".+?报([告表])?\\d{4}(\\.?\\d{1,2}(\\.?\\d{2})?)?");
        Pattern pat2 = Pattern.compile("私募.*披露年度报[告表](（\\d{4}-\\d{2}-\\d{2}至\\d{4}-\\d{2}-\\d{2}）)?");
        Pattern pat3 = Pattern.compile(".+?报([告表])?\\d{4}-\\d{2}-\\d{2}至\\d{4}-\\d{2}-\\d{2}?");

        // 创建Matcher对象
        Matcher matcher1 = pat1.matcher(text);
        Matcher matcher2 = pat2.matcher(text);
        Matcher matcher3 = pat3.matcher(text);

        // 尝试匹配
        String reportName;
        if (matcher1.find()) {
            reportName = matcher1.group();
        } else if (matcher2.find()) {
            reportName = matcher2.group();
        } else if (matcher3.find()) {
            reportName = matcher3.group();
        } else {
            reportName = text;
        }
        return reportName.replace("（", "(").replace("）", ")");
    }
}
